/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui;

/**************************************************************************************
 * Written:     04/07/03
 * Revised:     
 **************************************************************************************/
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
/** A JTextField that doesn't allow certain characters, either blocking or replacing them
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.4
 */
public class TransformCharacterTextField 
    extends JTextField {
 
    static private HashMap mappings = new HashMap();

    public TransformCharacterTextField() {
	super();
    }

    public TransformCharacterTextField(String text) {
	super(text);
    }
 
    public void blockCharacter(char x) {
	mappings.put(new Character(x), null);
    }

    public void replaceCharacter(char x, char y) {
	mappings.put(new Character(x), new Character(y));
    }

    public void restoreCharacter(char x) {
	mappings.remove(new Character(x));
    }

    protected Document createDefaultModel() {
	return new TransformCharacterTextDocument();
    }
 
    static class TransformCharacterTextDocument 
	extends PlainDocument {
		  
	public void insertString(int offs, String str, AttributeSet a) 
	    throws BadLocationException {
				
	    if (str == null) {
		return;
	    }
	    char[] raw = str.toCharArray();
	    StringBuffer result = new StringBuffer("");
	    for (int i = 0; i < raw.length; i++) {
		Character key = new Character(raw[i]);
		if(mappings.containsKey(key)) {
		    Character replacement = (Character) mappings.get(key);
		    if(replacement != null) {
			result.append(replacement.charValue());
			replacement = null;
		    }
		}
		else {
		    result.append(raw[i]);
		}
		key = null;
	    }
	    super.insertString(offs, result.toString(), a);
	}
    }
}
