package org.greenstone.gsdl3.gs3build.classifier;

import java.util.Iterator;

import org.greenstone.gsdl3.gs3build.doctypes.DocumentID;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentList;

public class ClassifierManager
{
	DocumentList           documents;
	ClassifierInterface [] list;
	int                    size;
	int                    used;

	public ClassifierManager(DocumentList documentList)
	{	this.list = new ClassifierInterface[10];
		this.size = 10;
		this.used = 0;
		this.documents = documentList;
	}

	/**
	 *  Add a classifier to the end of the list of classifiers...
	 *
	 *  @param <code>ClassifierInterface</code> the classifier.
	 */
	public void addClassifier(ClassifierInterface classifier)
	{	this.ensureSize(this.used + 1);
		this.list[this.used] = classifier;
		this.used ++;
	}

	public void classifyDocument(DocumentID docId, DocumentInterface document)
	{	for (int i = 0; i < this.used; i ++)
		{	this.list[i].classifyDocument(docId, document);
		}
	}

  /**
   *  Classify all the documents that the document list is able to provide...
   */
  public void classifyDocuments()
  { for (int i = 0; i < this.used; i ++)
    { for (int p = 0; p < this.list[i].getClassifierPasses(); p ++)
      {	this.list[i].startClassifierPass(p);

	Iterator iterator = this.documents.iterator();

	while (iterator.hasNext()) {
	  DocumentInterface document = (DocumentInterface) iterator.next();

	  if (document.isIndexed()) {
	    if (!this.list[i].classifyDocument(document.getID(), document)) {
	      System.out.println("Ending document");
	    }
	  }
	}
	this.list[i].endClassifierPass(p);
      }
      this.list[i].completeClassification();
    }
  }

  public void ensureSize(int size)
  { while (size >= this.size)
    { ClassifierInterface newList [] = new ClassifierInterface[this.size*2];
      this.size *= 2;
      System.arraycopy(this.list, 0, newList, 0, this.size);
      this.list = newList;
    }
  }
}
